// Strings.h
#ifndef Strings_h
#define Strings_h

#ifndef ssize_t
typedef SSIZE_T ssize_t;             // ssize_t is defined in GCC, but isn't defined in VC9-15, but rather SSIZE_T.  For symetry we'll define it.
#endif

#define MINIMUM_ALLOCATION     16    // allocate at least this for every String
#define EXPANSION_FACTOR        2    // repeated concatenations will keep doubling buffer
#define NUMERIC_CONVERSIONS          // allows direct assignment of numeric values to Strings, e.g., String s(3.14159) or s=3.14159;
#define FORMAT_MONEY                 // put commas every three places, rounding, left/right justification, specify field sizes (padding), etc.
#define CONSOLE_OUTPUT               // support console output, i.e., enable String::Print()
#define                       x64
//#define                   debug

class String
{
 public:                                           // Constructors (8)
 String();                                         // Uninitialized Constructor~String(); 
 String& operator=(const TCHAR* pStr);             // Assign a character string to a String Object, e.g.,  String s1 = "Compile Without Compromise";
 #ifdef CONSOLE_OUTPUT
    void Print(bool blnCrLf);                      // Outputs String with or without CrLf
    void Print(const TCHAR* pText, bool blnCrLf);  // Parameter #1 - leading text literal; Parameter #2 - with/without CrLf
 #endif
 int ParseCount(const TCHAR delimiter);                         // Returns count of delimited fields as specified by char delimiter, i.e., comma, space, tab, etc.
 void Parse(String* pStr, TCHAR delimiter, size_t iParseCount); // Parses this based on delimiter.  Must pass in 1st parameter String* to sufficient number of Strings
 void LTrim();                                                  // Removes leading white space by modifying existing String
 TCHAR* lpStr();                                                // Same as std::string.c_str().  Returns pointer to underlying Z String
 ~String();                                                     //String Destructor

 private:
 TCHAR*    lpBuffer;
 size_t    iLen;
 size_t    iCapacity; 
};
 
#endif
// End Strings.h
